/*jslint browser: true*/
/*global io, Pusher*/

var PUSHUPDATE = {

    socketConnected: false,

    pageTitle: document.title,

    preset: {
        'articleIdPrefix': 'pu-1215-article-',
        'sliceClassWrapper': 'pu-1215-slice-wrapper',
        'sliceClassUpdated': 'pu-1215-slice-wrapper-updated',
        'sliceIdPrefix': 'pu-1215-slice-'
    },

    /**
     * Adapter Pusher.com
     *
     * @param config object
     */
    initPusher: function (config) {

        'use strict';

        this.config = config;

        var pusher = new Pusher(this.config.key),
            channel = pusher.subscribe(this.config.channel),
            self = this;

        this.status('○ Use Pusher.com');

        pusher.connection.bind('connected', function () {
            self.status('• Use Pusher.com');
        });

        channel.bind('event', function (dataMsg) {
            var data = dataMsg.message;
            self.processData(data);
        });
    },

    /**
     * Adapter Socket.IO
     *
     * @param config object
     */
    initSocketIO: function (config) {

        'use strict';

        this.config = config;

        var socketio = io.connect(this.config.socketUrl),
            self = this;

        this.status('○ Use Socket IO');

        socketio.on('connect', function () {
            self.socketConnected = true;
            self.status('• Use Socket IO');
        });

        socketio.on('event', function (dataJson) {
            var data = JSON.parse(dataJson);
            self.processData(data);
        });
    },

    /**
     * empfangene Daten auswerten und weiter verarbeiten
     *
     * @param data object
     */
    processData: function (data) {

        'use strict';

        var auth = this.checkAuth(data.auth);

        if (!auth) {
            return false;
        }

        switch (data.action) {

        case 'slice_add':
            this.addSlice(data);
            break;

        case 'slice_update':
            this.updateSlice(data);
            break;

        case 'slice_moveup':
            this.moveSliceUp(data);
            break;

        case 'slice_movedown':
            this.moveSliceDown(data);
            break;

        case 'slice_delete':
            this.deleteSlice(data);
            break;

        default:
            this.log('no action specified');
            break;
        }
    },

    /**
     * Neuen Slice hinzufügen
     *
     * @param data object
     */
    addSlice: function (data) {

        'use strict';

        var elem_parent_id = this.preset.articleIdPrefix.concat(data.article_id, '-', data.ctype, '-', data.clang),
            elem_parent = document.getElementById(elem_parent_id),
            elems,
            elem,
            elem_next,
            elem_new = document.createElement('span');

        if (data.re_slice_id === 0) {
            // Slice an Position 1 setzen
            elems = elem_parent.childNodes;
            elem = elem_next = elems[0];
        } else {
            // Slice hinter re_slice_id setzen
            elem = this.selectSliceElem(data.re_slice_id, data.clang);
            elem_next = elem.nextSibling;
        }

        // Element erzeugen
        elem_new.className = this.preset.sliceClassWrapper;
        elem_new.id = this.preset.sliceIdPrefix.concat(data.slice_id, '-', data.clang);
        elem_new.innerHTML = data.content;

        // Element einfÃ¼gen
        elem_parent.insertBefore(elem_new, elem_next);

        this.setUpdated(elem_new, data.timestamp);

        this.callback(data);
    },

    /**
     * Inhalt in Content Slice ändern
     *
     * @param data object
     */
    updateSlice: function (data) {

        'use strict';

        var elem = this.selectSliceElem(data.slice_id, data.clang);
        elem.innerHTML = data.content;

        this.setUpdated(elem, data.timestamp);

        this.callback(data);
    },

    /**
     * Slice eine Position nach oben schieben
     *
     * @param data object
     */
    moveSliceUp: function (data) {

        'use strict';

        var elem = this.selectSliceElem(data.slice_id, data.clang),
            elem_parent = elem.parentNode,
            elem_prev = elem.previousSibling,
            elem_save = elem_parent.removeChild(elem);

        elem_parent.insertBefore(elem_save, elem_prev);

        this.setUpdated(elem_save, data.timestamp);

        this.callback(data);
    },

    /**
     * Slice eine Position nach unten schieben
     *
     * @param data object
     */
    moveSliceDown: function (data) {

        'use strict';

        var elem = this.selectSliceElem(data.slice_id, data.clang),
            elem_parent = elem.parentNode,
            elem_next = elem.nextSibling.nextSibling,
            elem_save = elem_parent.removeChild(elem);

        elem_parent.insertBefore(elem_save, elem_next);

        this.setUpdated(elem, data.timestamp);

        this.callback(data);
    },

    /**
     * Content Slice entfernen
     *
     * @param data object
     */
    deleteSlice: function (data) {

        'use strict';

        var elem = this.selectSliceElem(data.slice_id, data.clang),
            elem_parent = elem.parentNode;

        elem_parent.removeChild(elem);

        this.callback(data);
    },

    /**
     * Callback Funktion
     *
     * @param params object
     */
    callback: function (params) {

        'use strict';

        if (this.config.callback !== undefined) {
            delete params.auth;
            this.config.callback.call(params);
        }
    },

    /**
     * Element Id zusammensetzen und wählen
     *
     * @param slice_id int
     * @param clang int
     * @return object
     */
    selectSliceElem: function (slice_id, clang) {

        'use strict';

        var elemId = this.preset.sliceIdPrefix.concat(slice_id, '-', clang),
            elem = document.getElementById(elemId);

        return elem;
    },

    /**
     * Element als geupdated markieren
     *
     * @param elem object
     * @param time int
     */
    setUpdated: function (elem, time) {

        'use strict';

        elem.setAttribute('data-updatetime', time);

        this.removeClass(elem, this.preset.sliceClassUpdated);

        // triggering reflow
        // http://stackoverflow.com/questions/21664940/force-browser-to-trigger-reflow-while-changing-css
        elem.noop = elem.offsetHeight;

        this.addClass(elem, this.preset.sliceClassUpdated);
    },

    /**
     * Gesendeten Hash mit Cookie vergleichen
     *
     * @param auth string
     * @return boolean
     */
    checkAuth: function (auth) {

        'use strict';

        var cookie_name = 'rex-1215-auth',
            reg = new RegExp('[; ]' + cookie_name + '=([^\\s;]*)'),
            match = (' ' + document.cookie).match(reg);

        if (match && match[1] === auth) {
            return true;
        }

        return false;
    },

    /**
     * Testen ob Element CSS Klasse schon hat
     *
     * @param elem object
     * @param classname string
     * @return boolean
     */
    hasClass: function (elem, classname) {

        'use strict';

        var reg = new RegExp('(\\s|^)' + classname + '(\\s|$)');
        return elem.className.match(reg);
    },

    /**
     * CSS Klasse einem Element hinzufügen
     *
     * @param elem object
     * @param classname string
     */
    addClass: function (elem, classname) {

        'use strict';

        if (!this.hasClass(elem, classname)) {
            elem.className += ' ' + classname;
            elem.className.replace(/ +/g, ' ');
        }
    },

    /**
     * CSS Klasse aus Element entfernen
     *
     * @param elem object
     * @param classname string
     */
    removeClass: function (elem, classname) {

        'use strict';

        if (this.hasClass(elem, classname)) {
            var reg = new RegExp('(\\s|^)' + classname + '(\\s|$)');
            elem.className = elem.className.replace(reg, '');
        }
    },

    /**
     * Helper Console Log
     *
     * @param message mixed
     */
    log: function (message) {

        'use strict';

        if (window.console && window.console.log) {
            window.console.log(message);
        }
    },

    /**
     * Helper Status
     *
     * @param message mixed
     */
    status: function (text) {

        'use strict';

        document.title = ''.concat(this.pageTitle, ' | ', text);

    }
};

// ie8 Kompatibilität
if (!Object.create) {
    Object.create = (function () {

        'use strict';

        function F() {
            return;
        }

        return function (o) {
            F.prototype = o;
            return new F();
        };
    }());
}
